"""Base pagination iterator for traversing API respones."""


class BasePaginationIterator:

    def __init__(self, client, params=None, limit=None):
        self._client = client
        self.params = params or {}
        self.cursor = None
        self.has_more = True
        self.items = []
        self.limit = limit or 0
        if self.limit > 0:
            if self.limit < 50:
                self.params['limit'] = self.limit
        if limit not in self.params:
            self.params['limit'] = 50

    def parse_resp(self, resp):
        self.items = []

        try:
            self.items = self.parse_data(resp)
        except NotImplementedError:
            self.items = resp['data']

        pagination = resp['pagination']
        self.has_more = pagination['has_more']
        self.cursor = pagination['cursor']
        self.per_page = pagination['per_page']
        self.count = pagination['count']

    def do_req(self, params):
        raise NotImplementedError  # TODO(tsileo): make this one abstract

    def parse_data(self, data):
        """Custom function can return a list of dict/object that will be yield during iteration."""
        raise NotImplementedError

    def __iter__(self):
        return self

    def __next__(self):
        try:
            return self.items.pop(0)
        except IndexError:
            if not self.has_more:
                raise StopIteration
            params = self.params.copy()
            if self.cursor:
                params['cursor'] = self.cursor
            resp = self.do_req(params)
            self.parse_resp(resp)
            return next(self)



"""BlobStash client."""
import os
import json
from urllib.parse import urljoin


import requests


DEFAULT_BASE_URL = 'http://localhost:8050'


class Client:
    """Basic client for API-specific client to build upon."""

    def __init__(self, base_url=None, api_key=None, json_encoder=json.JSONEncoder):
        self.base_url = base_url or os.getenv('BLOBSTASH_BASE_URL', DEFAULT_BASE_URL)
        self.api_key = api_key or os.getenv('BLOBSTASH_API_KEY')
        self.json_encoder = json_encoder

    def request(self, verb, path, **kwargs):
        """Helper for making authenticated request to BlobStash."""
        raw = kwargs.pop('raw', False)
        json_data = kwargs.pop('json', None)
        if json_data:
            kwargs['data'] = json.dumps(json_data, cls=self.json_encoder)
            headers = kwargs.get('headers', {})
            headers['Content-Type'] = 'application/json'
            kwargs['headers'] = headers

        if self.api_key:
            kwargs['auth'] = ('', self.api_key)

        r = requests.request(
            verb,
            urljoin(self.base_url, path),
            **kwargs
        )
        if raw:
            return r

        r.raise_for_status()
        return r.json()



"""Utils for unit tests, also used for BlobStash (and related projects) integrations tests."""
from subprocess import Popen
import time
import shutil
import os


class BlobStash(object):
    """BlobStash process manager. Expects `blobstash` to be available in $PATH."""

    def __init__(self, config=None, rebuild=True):
        self.process = None
        self.config = config or 'blobstash.yaml'

    def run(self, reindex=False, log_level='error'):
        """Execute `blobsfs-mount {fs_name} {fs_name}` and return the running process."""
        cmd = ['blobstash', '--loglevel', log_level]
        if reindex:
            cmd.append('-scan')
        cmd.append(self.config)
        self.process = Popen(cmd, env=os.environ)
        time.sleep(1)
        if self.process.poll():
            raise Exception('failed to mount')

    def cleanup(self):
        """Cleanup func."""
        try:
            shutil.rmtree('blobstash_data')
        except:
            pass

    def shutdown(self):
        """Perform a clean shutdown."""
        if self.process:
            self.process.terminate()
            self.process.wait()
